@@ -36,6 +36,11 @@ extern char* GBLfsCurrentDir;
 extern bool GBLisoPaneActive;
 extern VolInfo GBLvolInfo;
 extern bool GBLisoChangesProbable;
+extern GtkWidget* GBLeditorFld;
+extern GtkWidget* GBLviewerFld;
+extern GtkWidget* GBLtempDirFld;
+extern GtkListStore* GBLfsListStore;
+extern GtkListStore* GBLisoListStore;
 
 void buildImagePropertiesWindow(GtkWidget *widget, GdkEvent *event)
 {
@@ -141,12 +146,40 @@ void buildImagePropertiesWindow(GtkWidget *widget, GdkEvent *event)
     {
         const char* publisher;
         const char* volName;
+        int rc2;
+        GtkWidget* warningDialog;
         
         publisher = gtk_entry_get_text(GTK_ENTRY(publisherField));
-        bk_set_publisher(&GBLvolInfo, publisher);
+        rc2 = bk_set_publisher(&GBLvolInfo, publisher);
+        if(rc2 <= 0)
+        {
+            warningDialog = gtk_message_dialog_new(GTK_WINDOW(GBLmainWindow),
+                                                   GTK_DIALOG_DESTROY_WITH_PARENT,
+                                                   GTK_MESSAGE_ERROR,
+                                                   GTK_BUTTONS_CLOSE,
+                                                   _("Invalid publisher '%s': '%s'"),
+                                                   publisher,
+                                                   bk_get_error_string(rc2));
+            gtk_window_set_modal(GTK_WINDOW(warningDialog), TRUE);
+            gtk_dialog_run(GTK_DIALOG(warningDialog));
+            gtk_widget_destroy(warningDialog);
+        }
         
         volName = gtk_entry_get_text(GTK_ENTRY(volNameField));
-        bk_set_vol_name(&GBLvolInfo, volName);
+        rc2 = bk_set_vol_name(&GBLvolInfo, volName);
+        if(rc2 <= 0)
+        {
+            warningDialog = gtk_message_dialog_new(GTK_WINDOW(GBLmainWindow),
+                                                   GTK_DIALOG_DESTROY_WITH_PARENT,
+                                                   GTK_MESSAGE_ERROR,
+                                                   GTK_BUTTONS_CLOSE,
+                                                   _("Invalid volume name '%s': '%s'"),
+                                                   publisher,
+                                                   bk_get_error_string(rc2));
+            gtk_window_set_modal(GTK_WINDOW(warningDialog), TRUE);
+            gtk_dialog_run(GTK_DIALOG(warningDialog));
+            gtk_widget_destroy(warningDialog);
+        }
         
         if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(rockridgeCheck)))
             GBLappSettings.filenameTypesToWrite |= FNTYPE_ROCKRIDGE;
@@ -164,6 +197,120 @@ void buildImagePropertiesWindow(GtkWidget *widget, GdkEvent *event)
     gtk_widget_destroy(dialog);
 }
 
+void changeEditorCbk(GtkButton *button, gpointer data)
+{
+    GtkWidget* dialog;
+    GtkWidget* textField;
+    int rc;
+    
+    dialog = gtk_dialog_new_with_buttons(_("Editor"),
+                                         GTK_WINDOW(GBLmainWindow),
+                                         GTK_DIALOG_DESTROY_WITH_PARENT,
+                                         GTK_STOCK_OK,
+                                         GTK_RESPONSE_ACCEPT,
+                                         GTK_STOCK_CANCEL,
+                                         GTK_RESPONSE_REJECT,
+                                         NULL);
+    g_signal_connect(dialog, "close", G_CALLBACK(rejectDialogCbk), NULL);
+    
+    textField = gtk_entry_new();
+    gtk_entry_set_text(GTK_ENTRY(textField), gtk_entry_get_text(GTK_ENTRY(GBLeditorFld)));
+    gtk_entry_set_width_chars(GTK_ENTRY(textField), 32);
+    g_signal_connect(textField, "activate", (GCallback)acceptDialogCbk, dialog);
+    gtk_widget_show(textField);
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), textField, TRUE, TRUE, 0);
+    
+    gtk_widget_show(dialog);
+    
+    rc = gtk_dialog_run(GTK_DIALOG(dialog));
+    if(rc == GTK_RESPONSE_ACCEPT)
+    {
+        const gchar* newEditor;
+        
+        newEditor = gtk_entry_get_text(GTK_ENTRY(textField));
+        
+        gtk_entry_set_text(GTK_ENTRY(GBLeditorFld), newEditor);
+    }
+    
+    gtk_widget_destroy(dialog);
+}
+
+void changeTempDirCbk(GtkButton *button, gpointer data)
+{
+    GtkWidget* dialog;
+    GtkWidget* textField;
+    int rc;
+    
+    dialog = gtk_dialog_new_with_buttons(_("Temporary directory"),
+                                         GTK_WINDOW(GBLmainWindow),
+                                         GTK_DIALOG_DESTROY_WITH_PARENT,
+                                         GTK_STOCK_OK,
+                                         GTK_RESPONSE_ACCEPT,
+                                         GTK_STOCK_CANCEL,
+                                         GTK_RESPONSE_REJECT,
+                                         NULL);
+    g_signal_connect(dialog, "close", G_CALLBACK(rejectDialogCbk), NULL);
+    
+    textField = gtk_entry_new();
+    gtk_entry_set_text(GTK_ENTRY(textField), gtk_entry_get_text(GTK_ENTRY(GBLtempDirFld)));
+    gtk_entry_set_width_chars(GTK_ENTRY(textField), 32);
+    g_signal_connect(textField, "activate", (GCallback)acceptDialogCbk, dialog);
+    gtk_widget_show(textField);
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), textField, TRUE, TRUE, 0);
+    
+    gtk_widget_show(dialog);
+    
+    rc = gtk_dialog_run(GTK_DIALOG(dialog));
+    if(rc == GTK_RESPONSE_ACCEPT)
+    {
+        const gchar* newDir;
+        
+        newDir = gtk_entry_get_text(GTK_ENTRY(textField));
+        
+        gtk_entry_set_text(GTK_ENTRY(GBLtempDirFld), newDir);
+    }
+    
+    gtk_widget_destroy(dialog);
+}
+
+void changeViewerCbk(GtkButton *button, gpointer data)
+{
+    GtkWidget* dialog;
+    GtkWidget* textField;
+    int rc;
+    
+    dialog = gtk_dialog_new_with_buttons(_("Viewer"),
+                                         GTK_WINDOW(GBLmainWindow),
+                                         GTK_DIALOG_DESTROY_WITH_PARENT,
+                                         GTK_STOCK_OK,
+                                         GTK_RESPONSE_ACCEPT,
+                                         GTK_STOCK_CANCEL,
+                                         GTK_RESPONSE_REJECT,
+                                         NULL);
+    g_signal_connect(dialog, "close", G_CALLBACK(rejectDialogCbk), NULL);
+    
+    textField = gtk_entry_new();
+    gtk_entry_set_text(GTK_ENTRY(textField), gtk_entry_get_text(GTK_ENTRY(GBLviewerFld)));
+    gtk_entry_set_width_chars(GTK_ENTRY(textField), 32);
+    g_signal_connect(textField, "activate", (GCallback)acceptDialogCbk, dialog);
+    gtk_widget_show(textField);
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), textField, TRUE, TRUE, 0);
+    
+    gtk_widget_show(dialog);
+    
+    rc = gtk_dialog_run(GTK_DIALOG(dialog));
+    if(rc == GTK_RESPONSE_ACCEPT)
+    {
+        const gchar* newViewer;
+        
+        newViewer = gtk_entry_get_text(GTK_ENTRY(textField));
+        
+        gtk_entry_set_text(GTK_ENTRY(GBLviewerFld), newViewer);
+    }
+    
+    gtk_widget_destroy(dialog);
+}
+
 void findHomeDir(void)
 {
     char* userHomeDir;
@@ -441,6 +588,135 @@ void loadSettings(void)
     /* no config file */
         GBLappSettings.appendExtension = true;
     
+    /* read/set editor */
+    if(GBLsettingsDictionary != NULL)
+    {
+        tempStr = iniparser_getstring(GBLsettingsDictionary, 
+                                      "ui:editor", NULL);
+        if(tempStr == NULL)
+        {
+            GBLappSettings.editor = malloc(strlen(DEFAULT_EDITOR) + 1);
+            if(GBLappSettings.editor == NULL)
+                fatalError("GBLappSettings.editor = malloc(strlen(DEFAULT_EDITOR) +1) failed");
+            strcpy(GBLappSettings.editor, DEFAULT_EDITOR);
+        }
+        else
+        {
+            /* pointer sharing is ok since GBLappSettings.editor is only written from here */
+            GBLappSettings.editor = tempStr;
+        }
+    }
+    else
+    /* no config file */
+    {
+        GBLappSettings.editor = malloc(strlen(DEFAULT_EDITOR) + 1);
+        if(GBLappSettings.editor == NULL)
+            fatalError("GBLappSettings.editor = malloc(strlen(DEFAULT_EDITOR) +1) failed");
+        strcpy(GBLappSettings.editor, DEFAULT_EDITOR);
+    }
+    
+    /* read/set viewer */
+    if(GBLsettingsDictionary != NULL)
+    {
+        tempStr = iniparser_getstring(GBLsettingsDictionary, 
+                                      "ui:viewer", NULL);
+        if(tempStr == NULL)
+        {
+            GBLappSettings.viewer = malloc(strlen(DEFAULT_VIEWER) + 1);
+            if(GBLappSettings.viewer == NULL)
+                fatalError("GBLappSettings.viewer = malloc(strlen(DEFAULT_VIEWER) +1) failed");
+            strcpy(GBLappSettings.viewer, DEFAULT_VIEWER);
+        }
+        else
+        {
+            /* pointer sharing is ok since GBLappSettings.viewer is only written from here */
+            GBLappSettings.viewer = tempStr;
+        }
+    }
+    else
+    /* no config file */
+    {
+        GBLappSettings.viewer = malloc(strlen(DEFAULT_VIEWER) + 1);
+        if(GBLappSettings.viewer == NULL)
+            fatalError("GBLappSettings.viewer = malloc(strlen(DEFAULT_VIEWER) +1) failed");
+        strcpy(GBLappSettings.viewer, DEFAULT_VIEWER);
+    }
+    
+    /* read/set temporary directory */
+    if(GBLsettingsDictionary != NULL)
+    {
+        tempStr = iniparser_getstring(GBLsettingsDictionary, 
+                                      "ui:tempdir", NULL);
+        if(tempStr == NULL)
+        {
+            GBLappSettings.tempDir = malloc(strlen(DEFAULT_TEMP_DIR) + 1);
+            if(GBLappSettings.tempDir == NULL)
+                fatalError("GBLappSettings.tempDir = malloc(strlen(DEFAULT_TEMP_DIR) +1) failed");
+            strcpy(GBLappSettings.tempDir, DEFAULT_TEMP_DIR);
+        }
+        else
+        {
+            /* pointer sharing is ok since GBLappSettings.tempDir is only written from here */
+            GBLappSettings.tempDir = tempStr;
+        }
+    }
+    else
+    /* no config file */
+    {
+        GBLappSettings.tempDir = malloc(strlen(DEFAULT_TEMP_DIR) + 1);
+        if(GBLappSettings.tempDir == NULL)
+            fatalError("GBLappSettings.tempDir = malloc(strlen(DEFAULT_TEMP_DIR) +1) failed");
+        strcpy(GBLappSettings.tempDir, DEFAULT_TEMP_DIR);
+    }
+    
+    /* read/set iso sort column id */
+    if(GBLsettingsDictionary != NULL)
+    {
+        GBLappSettings.isoSortColumnId = iniparser_getint(GBLsettingsDictionary, 
+                                                          "ui:isosortcolumnid", INT_MAX);
+        if(GBLappSettings.isoSortColumnId == INT_MAX)
+            GBLappSettings.isoSortColumnId = COLUMN_FILENAME;
+    }
+    else
+    /* no config file */
+        GBLappSettings.isoSortColumnId = COLUMN_FILENAME;
+    
+    /* read/set iso sort direction */
+    if(GBLsettingsDictionary != NULL)
+    {
+        GBLappSettings.isoSortDirection = iniparser_getint(GBLsettingsDictionary, 
+                                                          "ui:isosortdirection", INT_MAX);
+        if(GBLappSettings.isoSortDirection == INT_MAX)
+            GBLappSettings.isoSortDirection = GTK_SORT_ASCENDING;
+    }
+    else
+    /* no config file */
+        GBLappSettings.isoSortDirection = GTK_SORT_ASCENDING;
+    
+    /* read/set fs sort column id */
+    if(GBLsettingsDictionary != NULL)
+    {
+        GBLappSettings.fsSortColumnId = iniparser_getint(GBLsettingsDictionary, 
+                                                          "ui:fssortcolumnid", INT_MAX);
+        if(GBLappSettings.fsSortColumnId == INT_MAX)
+            GBLappSettings.fsSortColumnId = COLUMN_FILENAME;
+    }
+    else
+    /* no config file */
+        GBLappSettings.fsSortColumnId = COLUMN_FILENAME;
+    
+    /* read/set fs sort direction */
+    if(GBLsettingsDictionary != NULL)
+    {
+        GBLappSettings.fsSortDirection = iniparser_getint(GBLsettingsDictionary, 
+                                                          "ui:fssortdirection", INT_MAX);
+        if(GBLappSettings.fsSortDirection == INT_MAX)
+            GBLappSettings.fsSortDirection = GTK_SORT_ASCENDING;
+    }
+    else
+    /* no config file */
+        GBLappSettings.fsSortDirection = GTK_SORT_ASCENDING;
+    
     free(configFileName);
 }
 
@@ -470,6 +746,8 @@ void writeSettings(void)
     char numberStr[20];
     int width;
     int height;
+    int sortColumnId;
+    GtkSortType sortDirection;
     
     if(strcmp(GBLuserHomeDir, "/") == 0)
     {
@@ -537,5 +815,25 @@ void writeSettings(void)
     snprintf(numberStr, 20, "%d", GBLappSettings.appendExtension);
     iniparser_setstr(GBLsettingsDictionary, "ui:appendextension", numberStr);
     
+    iniparser_setstr(GBLsettingsDictionary, "ui:editor", gtk_entry_get_text(GTK_ENTRY(GBLeditorFld)));
+    
+    iniparser_setstr(GBLsettingsDictionary, "ui:viewer", gtk_entry_get_text(GTK_ENTRY(GBLviewerFld)));
+    
+    iniparser_setstr(GBLsettingsDictionary, "ui:tempdir", gtk_entry_get_text(GTK_ENTRY(GBLtempDirFld)));
+    
+    gtk_tree_sortable_get_sort_column_id(GTK_TREE_SORTABLE(GBLisoListStore), 
+                                         &sortColumnId, &sortDirection);
+    snprintf(numberStr, 20, "%d", sortColumnId);
+    iniparser_setstr(GBLsettingsDictionary, "ui:isosortcolumnid", numberStr);
+    snprintf(numberStr, 20, "%d", sortDirection);
+    iniparser_setstr(GBLsettingsDictionary, "ui:isosortdirection", numberStr);
+    
+    gtk_tree_sortable_get_sort_column_id(GTK_TREE_SORTABLE(GBLfsListStore), 
+                                         &sortColumnId, &sortDirection);
+    snprintf(numberStr, 20, "%d", sortColumnId);
+    iniparser_setstr(GBLsettingsDictionary, "ui:fssortcolumnid", numberStr);
+    snprintf(numberStr, 20, "%d", sortDirection);
+    iniparser_setstr(GBLsettingsDictionary, "ui:fssortdirection", numberStr);
+    
     iniparser_dump_ini(GBLsettingsDictionary, fileToWrite);
 }
